#include "Bitmapheightfield.h"
#include "PrimitiveObject.h"


BitmapHeightfield::BitmapHeightfield( dWorldID world, dSpaceID space, const char* file_name, float width, float depth, float h_scale )
    : m_hFieldWidth( width )
    , m_hFieldDepth( depth )
    , m_scale( h_scale )
{
	m_body = NULL;

    height_map = const_cast<unsigned char*>( createHeightMapFromBitmap( file_name, &m_stepWidth, &m_stepDepth ) );
    if (!height_map) {
        printf("NG\n");
        exit(1);
    }

    dHeightfieldDataID heightid = dGeomHeightfieldDataCreate();
    dGeomHeightfieldDataBuildByte(
        heightid,
        height_map,
        1,
        m_hFieldWidth,  // width
        m_hFieldDepth,  // depth
        m_stepWidth, // width steps
        m_stepDepth, // depth steps
        1.0/256.0*m_scale,
        0.0,
        0.0,
        0
    );
	//dGeomHeightfieldDataSetBounds( heightid, REAL( -100.0 ), REAL( +100.0 ) );

	m_geom = dCreateHeightfield( space, heightid, 1 );

	dMatrix3 R;
	dRSetIdentity( R );
	dRFromAxisAndAngle( R, 1, 0, 0, 90.0*M_PI/180.0 );  // オyオzɕϊ(ODE)
	dGeomSetRotation( m_geom, R );

	renderInit();
}


void BitmapHeightfield::render()
{
	//glPushMatrix();

    if (m_geom) {
	    const dReal *p = dGeomGetPosition(m_geom);
	    const dReal *r = dGeomGetRotation(m_geom);
	    GLfloat m[16];
	    m[ 0] = (GLfloat)r[ 0]; m[ 1] = (GLfloat)r[ 4]; m[ 2] = (GLfloat)r[ 8]; m[ 3] = (GLfloat)0;
	    m[ 4] = (GLfloat)r[ 1]; m[ 5] = (GLfloat)r[ 5]; m[ 6] = (GLfloat)r[ 9]; m[ 7] = (GLfloat)0;
	    m[ 8] = (GLfloat)r[ 2]; m[ 9] = (GLfloat)r[ 6]; m[10] = (GLfloat)r[10]; m[11] = (GLfloat)0;
	    m[12] = (GLfloat)p[ 0]; m[13] = (GLfloat)p[ 1]; m[14] = (GLfloat)p[ 2]; m[15] = (GLfloat)1;
	    glMultMatrixf( m );
    }
	
	renderInLocalFrame();

	//glPopMatrix();
}

void BitmapHeightfield::renderShadow()
{
    // nothing
}


void BitmapHeightfield::setColor(GLfloat r, GLfloat g, GLfloat b, GLfloat a)
{
	color[0] = r;
	color[1] = g;
	color[2] = b;
	color[3] = a;
}


void BitmapHeightfield::renderInLocalFrame() const
{
	static const GLfloat amb[]  = {0.8f, 0.8f, 0.8f, 1.0f};
	static const GLfloat spec[] = {1.0f, 1.0f, 1.0f, 1.0f};
	static const GLfloat shin[] = {1.0f};
    static const GLfloat shadow_spec[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    static const GLfloat shadow_diff[] = { 0.0f, 0.0f, 0.0f, 0.3f };

    if (m_flagShadow) {
	    glMaterialfv(GL_FRONT, GL_SPECULAR, shadow_spec);
	    glMaterialfv(GL_FRONT, GL_DIFFUSE, shadow_diff);
    }
    else {
	    glMaterialfv(GL_FRONT, GL_AMBIENT, amb);
	    glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
	    glMaterialfv(GL_FRONT, GL_SHININESS, shin);
	    glMaterialfv(GL_FRONT, GL_DIFFUSE, color);
    }

	glCallList(list);
}

void BitmapHeightfield::renderInit()
{
    list = glGenLists(1);
    glNewList(list, GL_COMPILE);
    {
        const float scale = (float)(1.0/256.0*m_scale);
        const float wsamp = ( m_hFieldWidth / ( m_stepWidth-1 ) );
        const float dsamp = ( m_hFieldDepth / ( m_stepDepth-1 ) );

        int ox = (int) ( -(int)m_hFieldWidth/2 );
        int oz = (int) ( -(int)m_hFieldDepth/2 );

	    for ( int i = 0; i < m_stepWidth - 1; ++i )
	    for ( int j = 0; j < m_stepDepth - 1; ++j )
	    {
		    float a[3], b[3], c[3], d[3];

		    a[ 0 ] = ox + ( i ) * wsamp;
            a[ 1 ] = (float)(height_map[(i+0)+(j+0)*m_stepWidth]) * scale;
		    a[ 2 ] = oz + ( j ) * dsamp;

		    b[ 0 ] = ox + ( i + 1 ) * wsamp;
            b[ 1 ] = (float)(height_map[(i+1)+(j+0)*m_stepWidth]) * scale;
		    b[ 2 ] = oz + ( j ) * dsamp;

		    c[ 0 ] = ox + ( i ) * wsamp;
            c[ 1 ] = (float)(height_map[(i+0)+(j+1)*m_stepWidth]) * scale;
		    c[ 2 ] = oz + ( j + 1 ) * dsamp;

		    d[ 0 ] = ox + ( i + 1 ) * wsamp;
            d[ 1 ] = (float)(height_map[(i+1)+(j+1)*m_stepWidth]) * scale;
		    d[ 2 ] = oz + ( j + 1 ) * dsamp;

            drawPrimTriangle( a, c, b, 1 );
            drawPrimTriangle( b, c, d, 1 );
	    }
    }
    glEndList();
}


// rbg}bv獂}bv𐶐
//   file_name : t@C(.bmp)
//   x_size    : rbg}bv̉TCYԂ
//   y_size    : rbg}bv̏cTCYԂ
//   return    : rbg}bvf[^̃|C^(擾sNULLԂ) x_size*y_sizë̗m
const unsigned char* BitmapHeightfield::createHeightMapFromBitmap( const char* file_name, int* x_size, int* y_size )
{
    unsigned char* height_map = NULL;

    unsigned short  bfType;
    unsigned long bfSize;
    unsigned short  bfReserved1;
    unsigned short  bfReserved2;
    unsigned long bfOffBits;
    unsigned long biSize;
    long  biWidth;
    long  biHeight;
    unsigned short  biPlanes;
    unsigned short  biBitCount;
    unsigned long biCompression;
    unsigned long biSizeImage;
    long  biXPelsPerMeter;
    long  biYPelsPerMeter;
    unsigned long biClrUsed;
    unsigned long biClrImportant;

    FILE *fp;
    fp = fopen( file_name, "rb" );
    if (fp == NULL) {
        printf("can not file open : %s\n", file_name );
        return NULL;
    }

    fread( &bfType, sizeof(unsigned short), 1, fp );
    fread( &bfSize, sizeof(unsigned long), 1, fp );
    fread( &bfReserved1, sizeof(unsigned short), 1, fp );
    fread( &bfReserved2, sizeof(unsigned short), 1, fp );
    fread( &bfOffBits, sizeof(unsigned long), 1, fp );
    fread( &biSize, sizeof(unsigned long), 1, fp );
    fread( &biWidth, sizeof(long), 1, fp );
    fread( &biHeight, sizeof(long), 1, fp );
    fread( &biPlanes, sizeof(unsigned short), 1, fp );
    fread( &biBitCount, sizeof(unsigned short), 1, fp );
    fread( &biCompression, sizeof(unsigned long), 1, fp );
    fread( &biSizeImage, sizeof(unsigned long), 1, fp );
    fread( &biXPelsPerMeter, sizeof(long), 1, fp );
    fread( &biYPelsPerMeter, sizeof(long), 1, fp );
    fread( &biClrUsed, sizeof(unsigned long), 1, fp );
    fread( &biClrImportant, sizeof(unsigned long), 1, fp );

    int pad = (biWidth*3+3)/4 * 4 - biWidth*3;
    int imgsize = (biWidth*3 + pad) * biHeight;

    height_map = new unsigned char[biWidth*biHeight];

    unsigned char *buf = new unsigned char[imgsize];
    fread( buf, 1, imgsize, fp );

    int cnt = 0;
    for (int y=biHeight-1; y>=0; y--) {
        for (int x=0; x<biWidth; x++) {
            int data = 0;
            int idx = y*(biWidth*3+pad) + x*3;
            data += buf[idx+0];
            data += buf[idx+1];
            data += buf[idx+2];
            data /= 3; // average
            height_map[cnt] = data;
            cnt++;
        }
    }
    delete [] buf;

    fclose( fp );

    *x_size = biWidth;
    *y_size = biHeight;

    return height_map;
}

