#include "ObjectBase.h"


ObjectBase::ObjectBase()
    : m_body(0)
    , m_geom(0)
    , m_flagShadow(false)
{
}

ObjectBase::~ObjectBase()
{
}

const dBodyID& ObjectBase::body() const
{
    return m_body;
}

const dGeomID& ObjectBase::geom() const
{
    return m_geom;
}


void ObjectBase::render()
{
	//glPushMatrix();

    if (m_body) {
	    const dReal *p = dBodyGetPosition(m_body);
	    const dReal *r = dBodyGetRotation(m_body);
	    GLfloat m[16];
	    m[ 0] = (GLfloat)r[ 0]; m[ 1] = (GLfloat)r[ 4]; m[ 2] = (GLfloat)r[ 8]; m[ 3] = (GLfloat)0;
	    m[ 4] = (GLfloat)r[ 1]; m[ 5] = (GLfloat)r[ 5]; m[ 6] = (GLfloat)r[ 9]; m[ 7] = (GLfloat)0;
	    m[ 8] = (GLfloat)r[ 2]; m[ 9] = (GLfloat)r[ 6]; m[10] = (GLfloat)r[10]; m[11] = (GLfloat)0;
	    m[12] = (GLfloat)p[ 0]; m[13] = (GLfloat)p[ 1]; m[14] = (GLfloat)p[ 2]; m[15] = (GLfloat)1;
	    glMultMatrixf( m );
    }
	
	renderInLocalFrame();

	//glPopMatrix();
}

void ObjectBase::renderShadow()
{
    m_flagShadow = true;

    render();

    m_flagShadow = false;
}

void ObjectBase::setPosition(dReal x, dReal y, dReal z)
{
    if (m_body)
    	dBodySetPosition(m_body, x, y, z);
    else if (m_geom)
    	dGeomSetPosition(m_geom, x, y, z);
}

const dReal* ObjectBase::getPosition() const
{
    if (m_body)
        return dBodyGetPosition(m_body);
    else if (m_geom)
        return dGeomGetPosition(m_geom);
    else
        return NULL;
}
