#ifndef __PRIMITIVE_OBJECT_H_
#define __PRIMITIVE_OBJECT_H_

#ifdef WIN32
#include <windows.h>
#endif

#include <math.h>
#ifndef M_PI
#define M_PI (3.14159265358979323846)
#endif

#include <GL/gl.h>


inline
void drawPrimPlane(float gsize)
{
    glBegin( GL_TRIANGLE_STRIP );
    glNormal3f( 0,0,1 );
    glVertex3f(-gsize,-gsize,0.0f);
    glVertex3f( gsize,-gsize,0.0f);
    glVertex3f(-gsize, gsize,0.0f);
    glVertex3f( gsize, gsize,0.0f);
    glEnd();
}


inline
void drawPrimSphere(GLfloat r, int lats, int longs)
{
    int i, j;
    for (i=0; i<=lats; i++) {
        GLfloat lat0 = (GLfloat)M_PI * (-0.5f + (GLfloat)(i-1)/(GLfloat)lats);
        GLfloat z0 = sin(lat0);
        GLfloat zr0 = cos(lat0);
 
        GLfloat lat1 = (GLfloat)M_PI * (-0.5f + (GLfloat)i/(GLfloat)lats);
        GLfloat z1 = sin(lat1);
        GLfloat zr1 = cos(lat1);
 
        glBegin(GL_QUAD_STRIP);
        for (j=0; j<=longs; j++) {
            GLfloat lng = 2.0f * (GLfloat)M_PI * (GLfloat)(j-1)/(GLfloat)longs;
            GLfloat x = cos(lng);
            GLfloat y = -sin(lng);
 
            glNormal3f(x * zr0, y * zr0, z0);
            glVertex3f(x * zr0, y * zr0, z0);
            glNormal3f(x * zr1, y * zr1, z1);
            glVertex3f(x * zr1, y * zr1, z1);
        }
        glEnd();
    }
}


inline
void drawPrimBox(float x, float y, float z)
{
	GLfloat lx = x * 0.5f;
    GLfloat ly = y * 0.5f;
    GLfloat lz = z * 0.5f;

    // sides
    glBegin (GL_TRIANGLE_STRIP);
    glNormal3f(-1,0,0);
    glVertex3f(-lx,-ly,-lz);
    glVertex3f(-lx,-ly, lz);
    glVertex3f(-lx, ly,-lz);
    glVertex3f(-lx, ly, lz);
    glNormal3f(0,1,0);
    glVertex3f( lx, ly,-lz);
    glVertex3f( lx, ly, lz);
    glNormal3f(1,0,0);
    glVertex3f( lx,-ly,-lz);
    glVertex3f( lx,-ly, lz);
    glNormal3f(0,-1,0);
    glVertex3f(-lx,-ly,-lz);
    glVertex3f(-lx,-ly, lz);
    glEnd();

    // top face
    glBegin (GL_TRIANGLE_FAN);
    glNormal3f (0,0,1);
    glVertex3f (-lx,-ly,lz);
    glVertex3f (lx,-ly,lz);
    glVertex3f (lx,ly,lz);
    glVertex3f (-lx,ly,lz);
    glEnd();

    // bottom face
    glBegin (GL_TRIANGLE_FAN);
    glNormal3f (0,0,-1);
    glVertex3f (-lx,-ly,-lz);
    glVertex3f (-lx,ly,-lz);
    glVertex3f (lx,ly,-lz);
    glVertex3f (lx,-ly,-lz);
    glEnd();
}

inline
void drawPrimCylinder(float r, float l, float zofs=0.0f, int quality=10 )
{
  int n = quality * 4;
  int i;
  float tmp,ny,nz,a,ca,sa;

  l *= 0.5;
  a = float(M_PI*2.0)/float(n);
  sa = (float) sin(a);
  ca = (float) cos(a);

  // draw cylinder body
  ny=1; nz=0;		  // normal vector = (0,ny,nz)
  glBegin (GL_TRIANGLE_STRIP);
  for (i=0; i<=n; i++) {
    glNormal3d (ny,nz,0);
    glVertex3d (ny*r,nz*r,l+zofs);
    glNormal3d (ny,nz,0);
    glVertex3d (ny*r,nz*r,-l+zofs);
    // rotate ny,nz
    tmp = ca*ny - sa*nz;
    nz = sa*ny + ca*nz;
    ny = tmp;
  }
  glEnd();

  // draw top cap
  //glShadeModel (GL_FLAT);
  ny=1; nz=0;		  // normal vector = (0,ny,nz)
  glBegin (GL_TRIANGLE_FAN);
  glNormal3d (0,0,1);
  glVertex3d (0,0,l+zofs);
  for (i=0; i<=n; i++) {
    glNormal3d (0,0,1);
    glVertex3d (ny*r,nz*r,l+zofs);

    // rotate ny,nz
    tmp = ca*ny - sa*nz;
    nz = sa*ny + ca*nz;
    ny = tmp;
  }
  glEnd();

  // draw bottom cap
  ny=1; nz=0;		  // normal vector = (0,ny,nz)
  glBegin (GL_TRIANGLE_FAN);
  glNormal3d (0,0,-1);
  glVertex3d (0,0,-l+zofs);
  for (i=0; i<=n; i++) {
    glNormal3d (0,0,-1);
    glVertex3d (ny*r,nz*r,-l+zofs);

    // rotate ny,nz
    tmp = ca*ny + sa*nz;
    nz = -sa*ny + ca*nz;
    ny = tmp;
  }
  glEnd();
}


inline
void drawPrimCapsule(float r, float l, int quality=3)
{
    int i,j;
    float tmp,nx,ny,nz,start_nx,start_ny,a,ca,sa;
    // number of sides to the cylinder (divisible by 4):
    int n = quality*4;
    
    l *= 0.5;
    a = float(M_PI*2.0)/float(n);
    sa = (float) sin(a);
    ca = (float) cos(a);
    
    // draw cylinder body
    ny=1; nz=0;		  // normal vector = (0,ny,nz)
    glBegin (GL_TRIANGLE_STRIP);
    for (i=0; i<=n; i++) {
        glNormal3d (ny,nz,0);
        glVertex3d (ny*r,nz*r,l);
        glNormal3d (ny,nz,0);
        glVertex3d (ny*r,nz*r,-l);
        // rotate ny,nz
        tmp = ca*ny - sa*nz;
        nz = sa*ny + ca*nz;
        ny = tmp;
    }
    glEnd();
    
    // draw first cylinder cap
    start_nx = 0;
    start_ny = 1;
    for (j=0; j<(n/4); j++) {
        // get start_n2 = rotated start_n
        float start_nx2 =  ca*start_nx + sa*start_ny;
        float start_ny2 = -sa*start_nx + ca*start_ny;
        // get n=start_n and n2=start_n2
        nx = start_nx; ny = start_ny; nz = 0;
        float nx2 = start_nx2, ny2 = start_ny2, nz2 = 0;
        glBegin (GL_TRIANGLE_STRIP);
        for (i=0; i<=n; i++) {
            glNormal3d (ny2,nz2,nx2);
                glVertex3d (ny2*r,nz2*r,l+nx2*r);
                glNormal3d (ny,nz,nx);
                glVertex3d (ny*r,nz*r,l+nx*r);
                // rotate n,n2
                tmp = ca*ny - sa*nz;
                nz = sa*ny + ca*nz;
                ny = tmp;
                tmp = ca*ny2- sa*nz2;
                nz2 = sa*ny2 + ca*nz2;
                ny2 = tmp;
        }
        glEnd();
        start_nx = start_nx2;
        start_ny = start_ny2;
    }
    
    // draw second cylinder cap
    start_nx = 0;
    start_ny = 1;
    for (j=0; j<(n/4); j++) {
        // get start_n2 = rotated start_n
        float start_nx2 = ca*start_nx - sa*start_ny;
        float start_ny2 = sa*start_nx + ca*start_ny;
        // get n=start_n and n2=start_n2
        nx = start_nx; ny = start_ny; nz = 0;
        float nx2 = start_nx2, ny2 = start_ny2, nz2 = 0;
        glBegin (GL_TRIANGLE_STRIP);
        for (i=0; i<=n; i++) {
            glNormal3d (ny,nz,nx);
            glVertex3d (ny*r,nz*r,-l+nx*r);
            glNormal3d (ny2,nz2,nx2);
            glVertex3d (ny2*r,nz2*r,-l+nx2*r);
            // rotate n,n2
            tmp = ca*ny - sa*nz;
            nz = sa*ny + ca*nz;
            ny = tmp;
            tmp = ca*ny2- sa*nz2;
            nz2 = sa*ny2 + ca*nz2;
            ny2 = tmp;
        }
        glEnd();
        start_nx = start_nx2;
        start_ny = start_ny2;
    }
}


inline
void drawPrimPyramid( float size )
{
    // draw the pyramid
    const float k = 0.03f;
    glBegin (GL_TRIANGLE_FAN);
    glNormal3f (0,-1,1);
    glVertex3f (0,0,size);
    glVertex3f (-size,-size,0);
    glVertex3f ( size,-size,0);
    glNormal3f (1,0,1);
    glVertex3f ( size, size,0);
    glNormal3f (0,1,1);
    glVertex3f (-size, size,0);
    glNormal3f (-1,0,1);
    glVertex3f (-size,-size,0);
    glEnd();
}


inline
void drawPrimTriangle (const float *v0, const float *v1, const float *v2, int solid)
{
  float u[3],v[3],normal[3];
  u[0] = float( v1[0] - v0[0] );
  u[1] = float( v1[1] - v0[1] );
  u[2] = float( v1[2] - v0[2] );
  v[0] = float( v2[0] - v0[0] );
  v[1] = float( v2[1] - v0[1] );
  v[2] = float( v2[2] - v0[2] );

  normal[0] = u[1]*v[2] - u[2]*v[1];
  normal[1] = u[2]*v[0] - u[0]*v[2];
  normal[2] = u[0]*v[1] - u[1]*v[0];

  float len = normal[0]*normal[0] + normal[1]*normal[1] + normal[2]*normal[2];
  if (len <= 0.0f) {
    normal[0] = 1;
    normal[1] = 0;
    normal[2] = 0;
  }
  else {
    len = 1.0f / (float)sqrt(len);
    normal[0] *= len;
    normal[1] *= len;
    normal[2] *= len;
  }

  glBegin(solid ? GL_TRIANGLES : GL_LINE_STRIP);
  glNormal3fv (normal);
  glVertex3fv (v0);
  glVertex3fv (v1);
  glVertex3fv (v2);
  glEnd();
}



#endif // end of file
