#include "SampleMonkey.h"
#include "PrimitiveObject.h"



SampleMonkey::SampleMonkey( dWorldID world, dSpaceID space, dReal mass)
{
    //stl = new ode_util::StlReader<float,dTriIndex>("box.stl");
    stl = new ode_utils::StlReader<float,dTriIndex>("monkey_binary.stl");

	m_body = dBodyCreate( world );

    dTriMeshDataID data;
    data = dGeomTriMeshDataCreate();
    dGeomTriMeshDataBuildSingle(data, stl->getVertices(), 3*sizeof(float), stl->getVertexCount(),
        stl->getIndices(), stl->getIndexCount(), 3*sizeof(dTriIndex));

    m_geom = dCreateTriMesh(space, data, 0, 0, 0);
    dGeomSetData( m_geom, data );

    dMass m;
    dMassSetTrimesh( &m, mass, m_geom );
    dGeomSetPosition( m_geom, -m.c[0], -m.c[1], -m.c[2] );
    dMassTranslate( &m, -m.c[0], -m.c[1], -m.c[2] );
    dBodySetMass( m_body, &m );

    dGeomSetBody( m_geom, m_body );
    
    renderInit();
}

SampleMonkey::~SampleMonkey()
{
    delete stl;
}

void SampleMonkey::setColor(GLfloat r, GLfloat g, GLfloat b, GLfloat a)
{
	color[0] = r;
	color[1] = g;
	color[2] = b;
	color[3] = a;
}

void SampleMonkey::renderInLocalFrame() const
{
    static const GLfloat amb[]  = {0.8f, 0.8f, 0.8f, 1.0f};
	static const GLfloat spec[] = {1.0f, 1.0f, 1.0f, 1.0f};
	static const GLfloat shin[] = {1.0f};
    static const GLfloat shadow_spec[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    static const GLfloat shadow_diff[] = { 0.0f, 0.0f, 0.0f, 0.3f };

    if (m_flagShadow) {
	    glMaterialfv(GL_FRONT, GL_SPECULAR, shadow_spec);
	    glMaterialfv(GL_FRONT, GL_DIFFUSE, shadow_diff);
    }
    else {
	    glMaterialfv(GL_FRONT, GL_AMBIENT, amb);
	    glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
	    glMaterialfv(GL_FRONT, GL_SHININESS, shin);
	    glMaterialfv(GL_FRONT, GL_DIFFUSE, color);
    }

    glShadeModel(GL_FLAT);
	glCallList(m_list);
}


void SampleMonkey::renderInit()
{
    m_list = glGenLists(1);
    glNewList(m_list, GL_COMPILE);
    {
        for (int ii = 0; ii < stl->getIndexCount()/3; ii++) {
            const float v[9] = {
                stl->getVertices()[stl->getIndices()[ii*3+0]*3 + 0],
                stl->getVertices()[stl->getIndices()[ii*3+0]*3 + 1],
                stl->getVertices()[stl->getIndices()[ii*3+0]*3 + 2],
                stl->getVertices()[stl->getIndices()[ii*3+1]*3 + 0],
                stl->getVertices()[stl->getIndices()[ii*3+1]*3 + 1],
                stl->getVertices()[stl->getIndices()[ii*3+1]*3 + 2],
                stl->getVertices()[stl->getIndices()[ii*3+2]*3 + 0],
                stl->getVertices()[stl->getIndices()[ii*3+2]*3 + 1],
                stl->getVertices()[stl->getIndices()[ii*3+2]*3 + 2]
            };
            drawPrimTriangle( &v[0], &v[3], &v[6], 1 );
        }
    }
    glEndList();
}

