#include "SimpleBox.h"
#include "PrimitiveObject.h"


SimpleBox::SimpleBox( dWorldID world, dSpaceID space, dReal x, dReal y, dReal z, dReal mass)
	: xsize((GLfloat)x)
	, ysize((GLfloat)y)
	, zsize((GLfloat)z)
{
	m_body = dBodyCreate( world );
	m_geom = dCreateBox( space, x, y, z );

	dMass m;
	dMassSetBox( &m, 1.0f, x, y, z );
	dMassAdjust( &m, mass );
	dGeomSetBody( m_geom, m_body );

	renderInit();
}

void SimpleBox::setColor(GLfloat r, GLfloat g, GLfloat b, GLfloat a)
{
	color[0] = r;
	color[1] = g;
	color[2] = b;
	color[3] = a;
}


void SimpleBox::renderInLocalFrame() const
{
	static const GLfloat amb[]  = {0.8f, 0.8f, 0.8f, 1.0f};
	static const GLfloat spec[] = {1.0f, 1.0f, 1.0f, 1.0f};
	static const GLfloat shin[] = {1.0f};
    static const GLfloat shadow_spec[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    static const GLfloat shadow_diff[] = { 0.0f, 0.0f, 0.0f, 0.3f };

    if (m_flagShadow) {
	    glMaterialfv(GL_FRONT, GL_SPECULAR, shadow_spec);
	    glMaterialfv(GL_FRONT, GL_DIFFUSE, shadow_diff);
    }
    else {
	    glMaterialfv(GL_FRONT, GL_AMBIENT, amb);
	    glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
	    glMaterialfv(GL_FRONT, GL_SHININESS, shin);
	    glMaterialfv(GL_FRONT, GL_DIFFUSE, color);
    }

	glCallList(theBox);
}

void SimpleBox::renderInit()
{
    theBox = glGenLists(1);
    glNewList(theBox, GL_COMPILE);
    {
        drawPrimBox( xsize, ysize, zsize );
    }
    glEndList();
}
