#include "SimpleCapsule.h"
#include "PrimitiveObject.h"


SimpleCapsule::SimpleCapsule( dWorldID world, dSpaceID space,dReal r,  dReal l, dReal mass)
    : length((GLfloat)l)
    , radius((GLfloat)r)
{
    m_body = dBodyCreate( world );
    m_geom = dCreateCCylinder( space, r, l );
    
    dMass m;
    dMassSetCapsule( &m, 1.0f, 1, r, l );
    dMassAdjust( &m, mass );
    dGeomSetBody( m_geom, m_body );
    
    renderInit();
}

void SimpleCapsule::setColor(GLfloat r, GLfloat g, GLfloat b, GLfloat a)
{
	color[0] = r;
	color[1] = g;
	color[2] = b;
	color[3] = a;
}

void SimpleCapsule::renderInLocalFrame() const
{
    static const GLfloat amb[]  = {0.8f, 0.8f, 0.8f, 1.0f};
	static const GLfloat spec[] = {0.5f, 0.5f, 0.5f, 1.0f};
    static const GLfloat shin[] = {10.0f};
    static const GLfloat shadow_spec[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    static const GLfloat shadow_diff[] = { 0.0f, 0.0f, 0.0f, 0.3f };

    if (m_flagShadow) {
	    glMaterialfv(GL_FRONT, GL_SPECULAR, shadow_spec);
	    glMaterialfv(GL_FRONT, GL_DIFFUSE, shadow_diff);
    }
    else {
        glMaterialfv(GL_FRONT, GL_AMBIENT, amb);
        glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
        glMaterialfv(GL_FRONT, GL_SHININESS, shin);
        glMaterialfv(GL_FRONT, GL_DIFFUSE, color);
    }
    
    glShadeModel(GL_SMOOTH);
    glCallList(theCapsule);
}

void SimpleCapsule::renderInit()
{
    theCapsule = glGenLists(1);
    glNewList(theCapsule, GL_COMPILE);
    {
        drawPrimCapsule( radius, length );
    }
    glEndList();
}

