#include "SimpleCylinder.h"
#include "PrimitiveObject.h"


SimpleCylinder::SimpleCylinder( dWorldID world, dSpaceID space,dReal r,  dReal l, dReal mass)
	: length(l)
	, radius(r)
	, zoffset(0.0f)
{
	m_body = dBodyCreate( world );
	m_geom = dCreateCylinder( space, r, l );

	dMass m;
	dMassSetCylinder( &m, 1.0f, 1, r, l );
	dMassAdjust( &m, mass );
	dGeomSetBody( m_geom, m_body );

	renderInit();
}

void SimpleCylinder::setColor(GLfloat r, GLfloat g, GLfloat b, GLfloat a)
{
	color[0] = r;
	color[1] = g;
	color[2] = b;
	color[3] = a;
}

void SimpleCylinder::renderInLocalFrame() const
{
    static const GLfloat amb[]  = {0.8f, 0.8f, 0.8f, 1.0f};
	static const GLfloat spec[] = {0.5f, 0.5f, 0.5f, 1.0f};
	static const GLfloat shin[] = {1.0f};
    static const GLfloat shadow_spec[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    static const GLfloat shadow_diff[] = { 0.0f, 0.0f, 0.0f, 0.3f };

    if (m_flagShadow) {
	    glMaterialfv(GL_FRONT, GL_SPECULAR, shadow_spec);
	    glMaterialfv(GL_FRONT, GL_DIFFUSE, shadow_diff);
    }
    else {
	    glMaterialfv(GL_FRONT, GL_AMBIENT, amb);
	    glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
        glMaterialfv(GL_FRONT, GL_DIFFUSE, color);
	    glMaterialfv(GL_FRONT, GL_SHININESS, shin);
    }

    glShadeModel(GL_SMOOTH);
	glCallList(theCylinder);
}

void SimpleCylinder::renderInit()
{
    theCylinder = glGenLists(1);
    glNewList(theCylinder, GL_COMPILE);
    {
        drawPrimCylinder( (GLfloat)radius, (GLfloat)length, (GLfloat)zoffset, 8 );
    }
    glEndList();
}
