#include "SimpleGround.h"
#include "PrimitiveObject.h"


SimpleGround::SimpleGround( dWorldID world, dSpaceID space, float gsize )
    : m_gsize( gsize )
{
    //m_body = dBodyCreate( world );
    m_geom = dCreatePlane( space, 0, 0, 1, 0 ); 

    renderInit();
}


void SimpleGround::renderInLocalFrame() const
{
    glCallList(m_ground_id);
    glCallList(m_marker_id);
}


void SimpleGround::renderInit()
{
    m_ground_id = glGenLists(1);
    glNewList(m_ground_id, GL_COMPILE);
    {
	    const GLfloat amb[]  = {0.1f, 0.1f, 0.1f, 1.0f};
	    const GLfloat diff[] = {0.4f, 0.3f, 0.2f, 1.0f};  // ground color
	    const GLfloat spec[] = {1.0f, 1.0f, 1.0f, 1.0f};
	    const GLfloat shin[] = {1.0f};
	    glMaterialfv(GL_FRONT, GL_AMBIENT, amb);
	    glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
	    glMaterialfv(GL_FRONT, GL_DIFFUSE, diff);
	    glMaterialfv(GL_FRONT, GL_SHININESS, shin);

	    // grid line
        glDisable(GL_LIGHTING);
        glColor3f(0.7f, 0.7f, 0.7f);
        glBegin(GL_LINES);
        for (int i = -(int)m_gsize; i <= (int)m_gsize; i++) {
            glVertex3f((GLfloat)i, -m_gsize, 0);
            glVertex3f((GLfloat)i,  m_gsize, 0);
            glVertex3f(-m_gsize, (GLfloat)i, 0);
            glVertex3f( m_gsize, (GLfloat)i, 0);
        }
        glEnd();
        glEnable(GL_LIGHTING);

        glEnable (GL_POLYGON_OFFSET_FILL);
        glPolygonOffset (1.0, 0);

        drawPrimPlane(m_gsize);

        glDisable (GL_POLYGON_OFFSET_FILL);
    }
    glEndList();

    m_marker_id = glGenLists(1);
    glNewList(m_marker_id, GL_COMPILE);
    {
	    const GLfloat amb[]  = {0.0f, 0.0f, 0.0f, 1.0f};
	    const GLfloat diff[] = {1.0f, 0.0f, 0.0f, 1.0f};  // marker color
	    const GLfloat spec[] = {1.0f, 1.0f, 1.0f, 1.0f};
	    const GLfloat shin[] = {1.0f};
	    glMaterialfv(GL_FRONT, GL_AMBIENT, amb);
	    glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
	    glMaterialfv(GL_FRONT, GL_DIFFUSE, diff);
	    glMaterialfv(GL_FRONT, GL_SHININESS, shin);

        drawPrimPyramid( 0.2f );
    }
    glEndList();
}
