#include "SimpleSphere.h"
#include "PrimitiveObject.h"


SimpleSphere::SimpleSphere( dWorldID world, dSpaceID space, dReal r, dReal mass)
	: radius( (GLfloat)r )
{
	m_body = dBodyCreate( world );
	m_geom = dCreateSphere( space, r );

	dMass m;
	dMassSetSphere( &m, 1.0f, r );
	dMassAdjust( &m, mass );
	dGeomSetBody( m_geom, m_body );

	renderInit();
}

void SimpleSphere::setColor(GLfloat r, GLfloat g, GLfloat b, GLfloat a)
{
	color[0] = r;
	color[1] = g;
	color[2] = b;
	color[3] = a;
}

void SimpleSphere::renderInLocalFrame() const
{
	static const GLfloat amb[]  = {0.8f, 0.8f, 0.8f, 1.0f};
	static const GLfloat spec[] = {0.1f, 0.1f, 0.1f, 1.0f};
	static const GLfloat shin[] = {50.0};
    static const GLfloat shadow_spec[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    static const GLfloat shadow_diff[] = { 0.0f, 0.0f, 0.0f, 0.3f };

    if (m_flagShadow) {
	    glMaterialfv(GL_FRONT, GL_SPECULAR, shadow_spec);
	    glMaterialfv(GL_FRONT, GL_DIFFUSE, shadow_diff);
    }
    else {
	    glMaterialfv(GL_FRONT, GL_AMBIENT, amb);
	    glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
	    glMaterialfv(GL_FRONT, GL_DIFFUSE, color);
	    glMaterialfv(GL_FRONT, GL_SHININESS, shin);
    }

    glShadeModel(GL_SMOOTH);
	glCallList(theSphere);
}


void SimpleSphere::renderInit()
{
    theSphere = glGenLists(1);
    glNewList(theSphere, GL_COMPILE);
    {
        drawPrimSphere( radius, 20, 20 );
    }
    glEndList();
}

