#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4996)
#endif

#include <vector>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "SimpleGround.h"
#include "SimpleSphere.h"
#include "SimpleBox.h"
#include "SimpleCylinder.h"
#include "SimpleCapsule.h"
#include "SampleBunny.h"
#include "SampleMonkey.h"
#include "BitmapHeightfield.h"

#include <GL/glui.h>
#include <GL/glut.h>
#include <ode/ode.h>

// for GLUT and GLUI
static int main_window;

enum {
    BTN_PAUSE = 100,
    TRN_EYE_XY,
    TRN_EYE_Z,
    TRN_CAMERA_XY,
    TRN_CAMERA_Z,
};

float rotation[16] = {
    1.0, 0.0, 0.0, 0.0,
    0.0, 1.0, 0.0, 0.0,
    0.0, 0.0, 1.0, 0.0,
    0.0, 0.0, 0.0, 1.0
};


// for lighting
static const GLfloat light_pos[]  = {0.0f, -1.0f, 10.0f, 0.0f};
static const GLfloat light_amb[]  = {0.1, 0.1, 0.1, 1.0};
static const GLfloat light_diff[] = {1.0, 1.0, 1.0, 1.0};
static const GLfloat light_spec[] = {0.3, 0.3, 0.3, 1.0};

// for Camera View
static float view_xyz[3] = {0,-30,0};
static float eye_xyz[3]  = {0, 0, 1.0f};
static float view_trans[3] = { 0.0f, -30.0f, 1.0f }; // angle1, angle2, dist
static float org_xyz[3]  = {view_xyz[0],view_xyz[1],view_xyz[2]};

// for ODE
static dWorldID world;
static dSpaceID space;
static dJointGroupID contactgroup;

static ObjectBase *ground;
static std::vector<ObjectBase*> objList;

static int flag_ode_sim_pause = 0;
static int flag_show_shadow = 1;
static int flag_show_eye = 0;


// Փˌop֐
static void nearCallback( void *data, dGeomID o1, dGeomID o2 )
{
	const int MAX_CONTACTS = 10;

	dBodyID b1 = dGeomGetBody( o1 ); // 1
	dBodyID b2 = dGeomGetBody( o2 ); // 2

	if ( b1 && b2 && dAreConnectedExcluding( b1, b2, dJointTypeContact ) )
	return; // ՓˑΏۂłȂ̂̏Փ˂͂͂

	dContact contact[MAX_CONTACTS];
	for ( int i=0; i<MAX_CONTACTS; i++ )
	{
        // ̓m̐ڐG̃p[^ݒ
        contact[i].surface.mode = dContactBounce | dContactSoftCFM | dContactSoftERP;
        contact[i].surface.mu = dInfinity;     // CW
        contact[i].surface.mu2 = 0;
        contact[i].surface.bounce = 0.5;       // W
        contact[i].surface.bounce_vel = 0.1;
        contact[i].surface.soft_cfm = 0.0001;  // CFMݒ
        contact[i].surface.soft_erp = 0.1;
	}

	// Փˌo
	int numc = dCollide( o1, o2, MAX_CONTACTS, &contact[0].geom, sizeof( dContact ) );
	if ( numc > 0 )
	{
		for ( int i=0; i<numc; i++ )
		{
			// Փ˂̔
			dJointID c = dJointCreateContact( world, contactgroup, contact+i );
			dJointAttach( c, b1, b2 );
		}
	}
}


void getShadowMatrix( float* mat, float pos_x, float pos_y, float pos_z )
{
    //shadow matrix
	for (int i=0; i<16; i++) mat[i] = 0;
 
	mat[0]=1.0f;
	mat[5]=1.0f;
	mat[8]=(pos_x-light_pos[0])/(light_pos[2]);
	mat[9]=(pos_y-light_pos[1])/(light_pos[2]);
	mat[15]=1.0f;
}


void setLight()
{
	glLightfv(GL_LIGHT0, GL_AMBIENT, light_amb);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diff);
	glLightfv(GL_LIGHT0, GL_SPECULAR, light_spec);
    glLightfv(GL_LIGHT0, GL_POSITION, light_pos);

    glEnable(GL_LIGHT0);
    glEnable(GL_LIGHTING);
}


void mGlutDisplay(void)
{
	///////////////////////////////////////////////////////////////////////////
	// ODE stepping

    if (!flag_ode_sim_pause) {
	    for (int i=0; i<10; i++){
		    dSpaceCollide( space, 0, &nearCallback );

		    dWorldStep( world, 0.001 );

		    dJointGroupEmpty( contactgroup ); 
	    }
    }

	///////////////////////////////////////////////////////////////////////////


    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

	// camera
    gluLookAt(view_xyz[0], view_xyz[1], view_xyz[2], eye_xyz[0], eye_xyz[1], eye_xyz[2], 0,0,1);

    glMultMatrixf(rotation);

	// lighting
	setLight();


	///////////////////////////////////////////////////////////////////////////
	// object drawing

    {
	    std::vector<ObjectBase*>::iterator it = objList.begin();
	    while( it != objList.end() )  // vf܂
	    {
            glPushMatrix();

		    (*it)->render();
		    ++it;                 // Ce[^Pi߂

            glPopMatrix();
	    }
        ground->render();
    }

    if (flag_show_eye == 1) {
        const float len = 0.5f;
        glDisable(GL_LIGHTING);
        glColor3f(1.0f, 0.0f, 0.3f);
        glBegin(GL_LINES);

        glVertex3f(eye_xyz[0]-len,eye_xyz[1],eye_xyz[2]);
        glVertex3f(eye_xyz[0]+len,eye_xyz[1],eye_xyz[2]);
        glVertex3f(eye_xyz[0],eye_xyz[1]-len,eye_xyz[2]);
        glVertex3f(eye_xyz[0],eye_xyz[1]+len,eye_xyz[2]);
        glVertex3f(eye_xyz[0],eye_xyz[1],eye_xyz[2]-len);
        glVertex3f(eye_xyz[0],eye_xyz[1],eye_xyz[2]+len);

        glEnd();
        glEnable(GL_LIGHTING);
    }

	///////////////////////////////////////////////////////////////////////////
    // shadow drawing

    if (flag_show_shadow == 1) {
        float mat[16];
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glDepthMask(GL_FALSE);

	    std::vector<ObjectBase*>::iterator it = objList.begin();
	    while( it != objList.end() )
	    {
            const dReal *pos = (*it)->getPosition();
            getShadowMatrix( mat, (float)pos[0], (float)pos[1], (float)pos[2] );
          
            glPushMatrix();

            glMultMatrixf(mat);
		    (*it)->renderShadow();

            glPopMatrix();
		    ++it;
	    }
        glDepthMask(GL_TRUE);
        glDisable(GL_BLEND);
    }

	///////////////////////////////////////////////////////////////////////////

    glutSwapBuffers();
    //glFlush();
}

void myGlutIdle(void)
{
    glutSetWindow(main_window);
	glutPostRedisplay();
}


void mGlutTimer(int value)
{
	glutPostRedisplay();
	glutTimerFunc(17, mGlutTimer , 0);
}


void mGlutResize(int w, int h)
{
    glViewport(0, 0, w, h);

    /* ϊs̐ݒ */
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    gluPerspective(30.0, (double)w / (double)h, 1.0, 100.0);

    /* fr[ϊs̐ݒ */
    glMatrixMode(GL_MODELVIEW);
    //glLoadIdentity();
}


void mGlutMouse(int button, int state, int x, int y)
{
    switch (button) {
    case GLUT_LEFT_BUTTON:
        break;
    case GLUT_RIGHT_BUTTON:
        break;
    case GLUT_MIDDLE_BUTTON:
        break;
    default:
        break;
    }
}


void mGlutKeyboard(unsigned char key, int x, int y)
{
    switch (key) {
        case 'q':
        case 'Q':
        case '\033':  /* '\033'  ESC  ASCII R[h */
            exit(0);
            break;
        default:
            break;
    }
    glutPostRedisplay();
}


void setView()
{
    float z_angle = -view_trans[0]*M_PI/180.0;
    float y_angle =  view_trans[1]*M_PI/180.0;
    float x_angle =  view_trans[1]*M_PI/180.0;

    float dist = sqrt(
         (view_xyz[0]-eye_xyz[0])*(view_xyz[0]-eye_xyz[0])
        +(view_xyz[1]-eye_xyz[1])*(view_xyz[1]-eye_xyz[1])
        +(view_xyz[2]-eye_xyz[2])*(view_xyz[2]-eye_xyz[2]) );

    float x0, y0, z0;
    float x1, y1, z1;
    float x2, y2, z2;
    float x3, y3, z3;
    float x4, y4, z4;

    x0 = org_xyz[0];
    y0 = org_xyz[1];
    z0 = org_xyz[2];

    x1 = ( x0 * cos( z_angle ) - y0 * sin( z_angle ) );
    y1 = ( x0 * sin( z_angle ) + y0 * cos( z_angle ) );
    z1 = 0;

    x2 = ( z1 * sin( y_angle ) + x1 * cos( y_angle ) );
    y2 = y1;
    z2 = ( z0 * cos( y_angle ) - x0 * sin( y_angle ) );

    x3 = x2;
    y3 = ( y2 * cos( x_angle ) - z2 * sin( x_angle ) );
    z3 = ( y0 * sin( x_angle ) + z0 * cos( x_angle ) );

    x4 = x3;
    y4 = y3;
    z4 = z0 + (z1 + z2 + z3);

    float zoom = (dist+view_trans[2]) / dist;
    //if (zoom < 0.1f) {
    //    zoom = 0.1f;
    //}
    view_xyz[0] = ( eye_xyz[0] + view_trans[2] * x4 );
    view_xyz[1] = ( eye_xyz[1] + view_trans[2] * y4 );
    view_xyz[2] = ( eye_xyz[2] + view_trans[2] * z4 );

    //printf("%f, v=%f,%f,%f\n", view_trans[2], view_xyz[0], view_xyz[1], view_xyz[2]);
}

void mGluiCallBack(int value)
{

    if (value == BTN_PAUSE) {
        flag_ode_sim_pause = !flag_ode_sim_pause;
    }
    else if (value == TRN_CAMERA_XY || value == TRN_CAMERA_Z) {
        setView();
    }
}


void init(void)
{
    glClearColor(0.5, 0.5, 1.0, 1.0);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);

    setView();
}


int main(int argc, char *argv[])
{
    int main_window_xpos = 50;
    int main_window_ypos = 50;
    int main_window_xsize = 500;
    int main_window_ysize = 400;

	//////////////////////////////////////////////////////////////////////////////
	// GLUT setting

    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
    glutInitWindowPosition(main_window_xpos, main_window_ypos);
	glutInitWindowSize(main_window_xsize, main_window_ysize);
    main_window = glutCreateWindow("test");

    glutDisplayFunc(mGlutDisplay);
    glutReshapeFunc(mGlutResize);
    glutMouseFunc(mGlutMouse);
    glutKeyboardFunc(mGlutKeyboard);
    //glutPassiveMotionFunc(mGlutMotion);
    glutTimerFunc(100 , mGlutTimer , 0);

    init();

	//////////////////////////////////////////////////////////////////////////////
	// ODE setting

	// ODE initialize
	//dInitODE();
    dInitODE2(0);

	world = dWorldCreate();
	dWorldSetGravity( world, 0.0, 0.0, -9.8 );
    dWorldSetCFM (world,1e-5);
    dWorldSetERP (world,0.1);
    dWorldSetAutoDisableFlag (world,1);
    dWorldSetLinearDamping(world, 0.00001);
    dWorldSetAngularDamping(world, 0.005);
    dWorldSetMaxAngularSpeed(world, 200);

	space = dHashSpaceCreate( 0 );
	contactgroup = dJointGroupCreate( 0 );

	// ground setting
	ground = (ObjectBase*)new SimpleGround( world, space, 30.0f );

	// ball setting
	SimpleSphere *ball1;
	ball1 = new SimpleSphere( world, space, 1.0f, 1.0f );
	ball1->setPosition( 0.0, 0.0, 10.0 );
	ball1->setColor( 1.0, 0.3, 0.3, 0.5 );
	objList.push_back( ball1 );

	// ball setting
	SimpleSphere *ball2;
	ball2 = new SimpleSphere( world, space, 1.0f, 1.0f );
	ball2->setPosition( 0.5, 0.0, 20.0 );
	ball2->setColor( 1.0, 0.3, 1.0 );
	objList.push_back( ball2 );

	//// box setting
	//SimpleBox *box1;
	//box1 = new SimpleBox( world, space, 2.0f, 1.8f, 2.5f, 1.0f );
	//box1->setPosition( 5.0, 0.0, 10.0 );
	//box1->setColor( 0.0, 1.0, 1.0 );
	//objList.push_back( box1 );

	//// cylinder setting
	//SimpleCylinder *cyl1;
	//cyl1 = new SimpleCylinder( world, space, 0.5f, 2.0f, 1.0f );
	//cyl1->setPosition( 5.0, 0.8, 5.0 );
	//cyl1->setColor( 1.0, 1.0, 0.0 ); 
	//objList.push_back( cyl1 );

	//// capped cylinder setting
	//SimpleCapsule *cap1;
	//cap1 = new SimpleCapsule( world, space, 0.5f, 2.0f, 1.0f );
	//cap1->setPosition( -1.0, 0.0, 4.0 );
	//cap1->setColor( 0.5, 1.0, 0.5 ); 
	//objList.push_back( cap1 );

 //   for (int i=0; i<10; i++) {
	//    SampleBunny *bunny1;
	//    // sample bunny setting
	//    bunny1 = new SampleBunny( world, space, 10.0f );
	//    bunny1->setPosition(-10.0+(float)i*2.0, 0.0, 2.0 );
	//    bunny1->setColor( 0.5, 1.0, 0.5 ); 
	//    objList.push_back( bunny1 );
 //   }

    SampleMonkey *monkey1;
    // sample monkey setting
    monkey1 = new SampleMonkey( world, space, 10.0f );
    monkey1->setPosition(3.0, 0.0, 5.0 );
    monkey1->setColor( 0.5, 0.0, 1.0 ); 
    objList.push_back( monkey1 );


    BitmapHeightfield *hf;
    hf = new BitmapHeightfield( world, space, "a.bmp", 10, 10, 1.0 ); // width, depth, height_scale
    hf->setPosition( 0.0, 0.0, 0.1 );
    hf->setColor(0.2, 0.2, 0.2 );
    objList.push_back( hf );


    dAllocateODEDataForThread(dAllocateMaskAll);


	//////////////////////////////////////////////////////////////////////////////
    // GLUI setting

    int glui_window_xpos = main_window_xpos + main_window_xsize + 50;
    int glui_window_ypos = main_window_ypos + main_window_ysize/2;

    GLUI *glui = GLUI_Master.create_glui(
        "GLUI TEST", 0, glui_window_xpos, glui_window_ypos );

    glui->add_column( false );
    GLUI_Translation *trans_eye_xy = glui->add_translation(
        "EYE XY", GLUI_TRANSLATION_XY, eye_xyz, TRN_EYE_XY, mGluiCallBack );
    trans_eye_xy->set_speed( 0.05f );
    trans_eye_xy->set_x( eye_xyz[0] );
    trans_eye_xy->set_y( eye_xyz[1] );

    glui->add_column( false );
    GLUI_Translation *trans_eye_z = glui->add_translation(
        "EYE Z", GLUI_TRANSLATION_Z, &eye_xyz[2], TRN_EYE_Z, mGluiCallBack );
    trans_eye_z->set_speed( 0.05f );
    trans_eye_z->set_z( eye_xyz[2] );

    glui->add_column( false );
    GLUI_Translation *trans_camera_xy = glui->add_translation(
        "Angle", GLUI_TRANSLATION_XY, view_trans, TRN_CAMERA_XY, mGluiCallBack );
    trans_camera_xy->set_speed( 0.5f );
    trans_camera_xy->set_x( view_trans[0] );
    trans_camera_xy->set_y( view_trans[1] );

    glui->add_column( false );
    GLUI_Translation *trans_camera_z = glui->add_translation(
        "Zoom", GLUI_TRANSLATION_Z, &view_trans[2], TRN_CAMERA_Z, mGluiCallBack );
    trans_camera_z->set_speed( 0.01f );
    trans_camera_z->set_z( view_trans[2] );

    glui->add_column( false );
    GLUI_Rotation *trans_rotation = glui->add_rotation( "Rotation", rotation );
    //trans_rotation->set_spin( 1.0 );


    glui->add_column( false );
    glui->add_button( "pause", BTN_PAUSE, mGluiCallBack );

    glui->add_checkbox( "shadow", &flag_show_shadow );
    glui->add_checkbox( "eye", &flag_show_eye );

    glui->add_column( false );
    GLUI_Button *button = glui->add_button( "quit", 0,(GLUI_Update_CB)exit );
    //button->set_alignment( GLUI_ALIGN_RIGHT );


    glui->set_main_gfx_window( main_window );
    GLUI_Master.set_glutIdleFunc( myGlutIdle );


	//////////////////////////////////////////////////////////////////////////////
	// simulation

    glutMainLoop();


	//////////////////////////////////////////////////////////////////////////////
    // finished

    dJointGroupDestroy (contactgroup);
    dSpaceDestroy (space);
    dWorldDestroy (world);
	dCloseODE();

    return 0;
}
